using System;
using System.Diagnostics;
using System.Xml;

namespace Bespoke.Robotics.BrainStem.Configuration
{
    /// <summary>
    /// The BrainStem Xml element.
    /// </summary>
    public class BrainStemElement
    {
        /// <summary>
        /// The name of the element.
        /// </summary>
        public const string ELEMENT_NAME = "brainStem";

        /// <summary>
        /// Gets the BrainStem module address.
        /// </summary>
		/// <value>The BrainStem module address.</value>
        public byte ModuleAddress
        {
            get
            {
                return mModuleAddress;
            }
        }

        /// <summary>
		/// Gets the amount of time to wait for a command reply.
        /// </summary>
		/// <value>The amount of time to wait for a command reply.</value>
        public int CommandReplyTimeout
        {
            get
            {
                return mCommandReplyTimeout;
            }
        }

        /// <summary>
		/// Initializes a new instance of the BrainStemElement class.
        /// </summary>
        /// <param name="configNode">An Xml node containing the BrainStem element.</param>
        public BrainStemElement(XmlNode configNode)
        {
            Clear();
            ParseConfig(configNode);
        }

        /// <summary>
		/// Reset all configuration data.
        /// </summary>
        public void Clear()
        {
            mModuleAddress = 2;
            mCommandReplyTimeout = 1000;
        }

        /// <summary>
		/// Parse the configuration file.
        /// </summary>
		/// <param name="configNode">An Xml node containing the BrainStem element.</param>
        /// <exception cref="Exception">Thrown if there are any errors reading the BrainStem element.</exception>
        private void ParseConfig(XmlNode configNode)
        {
            try
            {
                mModuleAddress = byte.Parse(configNode.Attributes[MODULEADDRESS_ATTRIBUTE].Value.Trim());
                mCommandReplyTimeout = int.Parse(configNode.Attributes[COMMANDREPLYTIMEOUT_ATTRIBUTE].Value.Trim());
            }
            catch (Exception ex)
            {
                Clear();
                Trace.WriteLine(ex.Message);
                throw ex;
            }
        }

        private const string MODULEADDRESS_ATTRIBUTE = "moduleAddress";
        private const string COMMANDREPLYTIMEOUT_ATTRIBUTE = "commandReplyTimeout";

        private byte mModuleAddress;
        private int mCommandReplyTimeout;
    }
}
