using System;
using System.Diagnostics;
using System.Xml;

namespace Bespoke.Robotics.BrainStem.Configuration
{
    /// <summary>
    /// Exposes the BrainStem configuration file through a set of objects.
    /// </summary>
    public class Configuration
    {
        /// <summary>
        /// Gets the BrainStem configuration element.
        /// </summary>
		/// <value>The BrainStem configuration element.</value>
        public static BrainStemElement BrainStem
        {
            get
            {
                if (mIsInitialized == false)
                {
                    ParseConfig();
                }

                return mBrainStemElement;
            }
        }

        /// <summary>
        /// Gets the serial port configuration element.
        /// </summary>
		/// <value>The serial port configuration element.</value>
        public static SerialPortElement SerialPort
        {
            get
            {
                if (mIsInitialized == false)
                {
                    ParseConfig();
                }

                return mSerialPortElement;
            }
        }

        /// <summary>
        /// Reset all configuration data.
        /// </summary>
        public static void Clear()
        {
			mBrainStemElement = null;
            mSerialPortElement = null;
            mIsInitialized = false;
        }

        /// <summary>
        /// Parse the configuration file.
        /// </summary>
        private static void ParseConfig()
        {
            try
            {
                XmlDocument configDocument = new XmlDocument();
                configDocument.Load(CONFIG_FILENAME);

                foreach (XmlNode childNode in configDocument.DocumentElement.ChildNodes)
                {
                    switch (childNode.Name)
                    {
                        case BrainStemElement.ELEMENT_NAME:
                            mBrainStemElement = new BrainStemElement(childNode);
                            break;

                        case SerialPortElement.ELEMENT_NAME:
                            mSerialPortElement = new SerialPortElement(childNode);
                            break;
                    }
                }

                mIsInitialized = true;
            }
            catch (Exception ex)
            {
                Clear();
                Trace.WriteLine(ex.Message);
                throw ex;
            }
        }

        private static string CONFIG_FILENAME = @"Bespoke.Robotics.BrainStem.config";

        private static bool mIsInitialized;
        private static BrainStemElement mBrainStemElement;
        private static SerialPortElement mSerialPortElement;
    }
}
