using System;

namespace Bespoke.Robotics.BrainStem
{
	/// <summary>
	/// 
	/// </summary>
	public class DigitalIoPinConfiguration
	{
		/// <summary>
		/// 
		/// </summary>
		public DigitalIoPinType DigitalIoPin
		{
			get
			{
				return mDigitalIoPin;
			}
			set
			{
				mDigitalIoPin = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public DigitalIoDirectionType Direction
		{
			get
			{
				return mDirection;
			}
			set
			{
				mDirection = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public bool Enabled
		{
			get
			{
				return mEnabled;
			}
			set
			{
				mEnabled = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public bool Trigger
		{
			get
			{
				return mTrigger;
			}
			set
			{
				mTrigger = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public bool TargetState
		{
			get
			{
				return mTargetState;
			}
			set
			{
				mTargetState = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		public DigitalIoTimerModeType TimerMode
		{
			get
			{
				return mTimerMode;
			}
			set
			{
				mTimerMode = value;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="digitalIoPin"></param>
		public DigitalIoPinConfiguration(DigitalIoPinType digitalIoPin)
		{
			mDigitalIoPin = digitalIoPin;
			mDirection = DigitalIoDirectionType.Input;
			mEnabled = false;
			mTrigger = false;
			mTargetState = false;
			mTimerMode = DigitalIoTimerModeType.Edge;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="digitalIoPin"></param>
		/// <param name="direction"></param>
		/// <param name="enabled"></param>
		/// <param name="trigger"></param>
		/// <param name="targetState"></param>
		/// <param name="timerMode"></param>
		public DigitalIoPinConfiguration(DigitalIoPinType digitalIoPin, DigitalIoDirectionType direction,
			bool enabled, bool trigger, bool targetState, DigitalIoTimerModeType timerMode)
		{
			mDigitalIoPin = digitalIoPin;
			mDirection = direction;
			mEnabled = enabled;
			mTrigger = trigger;
			mTargetState = targetState;
			mTimerMode = timerMode;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="digitalIoPin">x</param>
		/// <param name="configurationFlags"></param>
		public DigitalIoPinConfiguration(DigitalIoPinType digitalIoPin, DigitalIoPinConfigurationFlags configurationFlags)
		{
			mDigitalIoPin = digitalIoPin;

			if (DigitalIoPinConfiguration.IsFlagOn(configurationFlags, DigitalIoPinConfigurationFlags.Direction))
			{
				mDirection = DigitalIoDirectionType.Output;
			}
			else
			{
				mDirection = DigitalIoDirectionType.Input;
			}

			mEnabled = DigitalIoPinConfiguration.IsFlagOn(configurationFlags, DigitalIoPinConfigurationFlags.Enabled);
			mTrigger = DigitalIoPinConfiguration.IsFlagOn(configurationFlags, DigitalIoPinConfigurationFlags.Trigger);
			mTargetState = DigitalIoPinConfiguration.IsFlagOn(configurationFlags, DigitalIoPinConfigurationFlags.TargetState);

			if (DigitalIoPinConfiguration.IsFlagOn(configurationFlags, DigitalIoPinConfigurationFlags.TimerMode))
			{
				mTimerMode = DigitalIoTimerModeType.Width;
			}
			else
			{
				mTimerMode = DigitalIoTimerModeType.Edge;
			}
		}

		/// <summary>
		/// 
		/// </summary>
		/// <returns></returns>
		public byte ToDataByte()
		{
			byte dataByte = 0;
			if (mDirection == DigitalIoDirectionType.Output)
			{
				dataByte |= (byte)DigitalIoPinConfigurationFlags.Direction;
			}

			if (mEnabled)
			{
				dataByte |= (byte)DigitalIoPinConfigurationFlags.Enabled;
			}

			if (mTrigger)
			{
				dataByte |= (byte)DigitalIoPinConfigurationFlags.Trigger;
			}

			if (mTargetState)
			{
				dataByte |= (byte)DigitalIoPinConfigurationFlags.TargetState;
			}

			if (mTimerMode == DigitalIoTimerModeType.Width)
			{
				dataByte |= (byte)DigitalIoPinConfigurationFlags.TimerMode;
			}

			return dataByte;
		}

		/// <summary>
		/// 
		/// </summary>
		/// <param name="flags"></param>
		/// <param name="flagToTest"></param>
		/// <returns></returns>
		public static bool IsFlagOn(DigitalIoPinConfigurationFlags flags, DigitalIoPinConfigurationFlags flagToTest)
		{
			return ((flags & flagToTest) != 0);
		}

		private DigitalIoPinType mDigitalIoPin;
		private DigitalIoDirectionType mDirection;
		private bool mEnabled;
		private bool mTrigger;
		private bool mTargetState;
		private DigitalIoTimerModeType mTimerMode;
	}
}
