using System;

namespace Bespoke.Robotics.BrainStem.Exceptions
{
    /// <summary>
    /// Describes general command exceptions.
    /// </summary>
    public class CommandException : Exception
    {
		/// <summary>
		/// Gets the associated command.
		/// </summary>
		/// <value>The associated command.</value>
		public Packet.CommandCodeType CommandCode
		{
			get
			{
				return mCommandCode;
			}
		}

		/// <summary>
		/// Gets the associated message packet.
		/// </summary>
		/// <value>The associated message packet.</value>
		public Packet MessagePacket
		{
			get
			{
				return mMessagePacket;
			}
		}

		/// <summary>
		/// Initializes a new instance of the CommandException class.
		/// </summary>
		/// <param name="commandCode">The associated command.</param>
		/// <param name="message">The message that describes the error.</param>
		public CommandException(Packet.CommandCodeType commandCode, string message)
			: this(commandCode, message, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the CommandException class.
		/// </summary>
		/// <param name="commandCode">The associated command.</param>
		/// <param name="messagePacket">The associated message packet.</param>
		public CommandException(Packet.CommandCodeType commandCode, Packet messagePacket)
			: this(commandCode, String.Empty, messagePacket)
		{
		}

		/// <summary>
		/// Initializes a new instance of the CommandException class.
		/// </summary>
		/// <param name="commandCode">The associated command.</param>
		/// <param name="message">The message that describes the error.</param>
		/// <param name="messagePacket">The associated message packet.</param>
		public CommandException(Packet.CommandCodeType commandCode, string message, Packet messagePacket)
			: base(message)
		{
			mCommandCode = commandCode;
			mMessagePacket = messagePacket;
		}

		private Packet.CommandCodeType mCommandCode;
		private Packet mMessagePacket;
    }
}
